// TutorialView.cpp : implementation of the CTutorialView class
//

#include "stdafx.h"
#include "Tutorial.h"

#include "TutorialDoc.h"
#include "TutorialView.h"

#include "Cube.h"
#include "Math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTutorialView

IMPLEMENT_DYNCREATE(CTutorialView, CView)

BEGIN_MESSAGE_MAP(CTutorialView, CView)
	//{{AFX_MSG_MAP(CTutorialView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTutorialView construction/destruction

CTutorialView::CTutorialView()
{
	// TODO: add construction code here
	m_hGLContext = NULL;
	m_GLPixelIndex = 0;
	m_opengl_init = false;
	for (int x = 0 ; x < NB_CUBE_X ; x++)
		for (int y = 0 ; y < NB_CUBE_Y ; y++)
			for (int z = 0 ; z < NB_CUBE_Z ; z++)
				Cube[x][y][z] = NULL;
	((CTutorialApp *)AfxGetApp())->pView = this;

}

CTutorialView::~CTutorialView()
{
	for (int x = 0 ; x < NB_CUBE_X ; x++)
		for (int y = 0 ; y < NB_CUBE_Y ; y++)
			for (int z = 0 ; z < NB_CUBE_Z ; z++)
				if (Cube[x][y][z])
				{
					delete Cube[x][y][z];
					Cube[x][y][z] = NULL;
				}
}

BOOL CTutorialView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	cs.cx = 640;
	cs.cy = 480;
	cs.style |= (WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
	cs.lpszClass = AfxRegisterWndClass(CS_OWNDC);

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTutorialView drawing

void CTutorialView::OnDraw(CDC* pDC)
{
	CTutorialDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CTutorialView diagnostics

#ifdef _DEBUG
void CTutorialView::AssertValid() const
{
	CView::AssertValid();
}

void CTutorialView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTutorialDoc* CTutorialView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTutorialDoc)));
	return (CTutorialDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTutorialView message handlers

BOOL CTutorialView::SetWindowPixelFormat(HDC hDC)
{
	PIXELFORMATDESCRIPTOR pixelDesc;

	pixelDesc.nSize		= sizeof(PIXELFORMATDESCRIPTOR);
	pixelDesc.nVersion	= 1;

	pixelDesc.dwFlags	=	PFD_DRAW_TO_WINDOW | 
							PFD_SUPPORT_OPENGL | 
							PFD_DOUBLEBUFFER |
							PFD_STEREO_DONTCARE;

	pixelDesc.iPixelType		= PFD_TYPE_RGBA;
	pixelDesc.cColorBits		= 32;
	pixelDesc.cRedBits			= 8;
	pixelDesc.cRedShift			= 16;
	pixelDesc.cGreenBits		= 8;
	pixelDesc.cGreenShift		= 8;
	pixelDesc.cBlueBits			= 8;
	pixelDesc.cBlueShift		= 0;
	pixelDesc.cAlphaBits		= 0;
	pixelDesc.cAlphaShift		= 0;
	pixelDesc.cAccumBits		= 64;	
	pixelDesc.cAccumRedBits		= 16;
	pixelDesc.cAccumGreenBits	= 16;
	pixelDesc.cAccumBlueBits	= 16;
	pixelDesc.cAccumAlphaBits	= 0;
	pixelDesc.cDepthBits		= 32;
	pixelDesc.cStencilBits		= 8;
	pixelDesc.cAuxBuffers		= 0;
	pixelDesc.iLayerType		= PFD_MAIN_PLANE;
	pixelDesc.bReserved			= 0;
	pixelDesc.dwLayerMask		= 0;
	pixelDesc.dwVisibleMask		= 0;
	pixelDesc.dwDamageMask		= 0;

	m_GLPixelIndex = ChoosePixelFormat(hDC, &pixelDesc);
	if (m_GLPixelIndex==0)
	{
		m_GLPixelIndex = 1;	// Let's choose a default index.
		if (DescribePixelFormat( hDC, m_GLPixelIndex, sizeof(PIXELFORMATDESCRIPTOR), &pixelDesc) == 0)
		{
			DWORD errorCode = GetLastError();
			CString errorStr;
			errorStr.Format("DescribePixelFormat returned error code %d.", errorCode);
			AfxMessageBox(errorStr);
			return FALSE;
		}
	}

	if (SetPixelFormat(	hDC, m_GLPixelIndex, &pixelDesc) ==FALSE)
	{
		DWORD errorCode = GetLastError();
		CString errorStr;
		errorStr.Format("SetPixelFormat returned error code %d.", errorCode);
		AfxMessageBox(errorStr);
		return FALSE;
	}
	return TRUE;
}

int CTutorialView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	HWND hWnd = GetSafeHwnd();
	HDC hDC = ::GetDC(hWnd);

	if (SetWindowPixelFormat(hDC)==FALSE)
		return 0;

	if (CreateViewGLContext(hDC)==FALSE)
		return 0;

	for (int x = 0 ; x < NB_CUBE_X ; x++)
		for (int y = 0 ; y < NB_CUBE_Y ; y++)
			for (int z = 0 ; z < NB_CUBE_Z ; z++)
				Cube[x][y][z] = new CCube(CUBE_R, x*CUBE_R, -y*CUBE_R, -z*CUBE_R, 0, 0, 0, 0, 0, 0, 0.07, 0.03, 0.05, false, true, 150, "Marbre.bmp");
	return 0;
}
	
BOOL CTutorialView::CreateViewGLContext(HDC hDC)
{
	m_hGLContext = wglCreateContext(hDC);
	if (m_hGLContext == NULL)
	{
		DWORD errorCode = GetLastError();
		CString errorStr;
		errorStr.Format("wglCreateContext returned error code %d.", errorCode);
		AfxMessageBox(errorStr);
		return FALSE;
	}

	if (wglMakeCurrent(hDC, m_hGLContext)==FALSE)
	{
		DWORD errorCode = GetLastError();
		CString errorStr;
		errorStr.Format("wglCreateContext returned error code %d.", errorCode);
		AfxMessageBox(errorStr);
		return FALSE;
	}

	return TRUE;
}

void CTutorialView::OnDestroy() 
{
	m_opengl_init = false;
	if(wglGetCurrentContext()!=NULL) 
	{
		// make the rendering context not current
		wglMakeCurrent(NULL, NULL) ;
	}
	
	if (m_hGLContext!=NULL)
	{
		wglDeleteContext(m_hGLContext);
		m_hGLContext = NULL;
	}

	// Now the associated DC can be released.
	CView::OnDestroy();
	
	// TODO: Add your message handler code here
	
}

void CTutorialView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	
	GLsizei width, height;
	GLdouble aspect;

	width = cx;
	height = cy;

	if (cy==0)
		aspect = (GLdouble)width;
	else
		aspect = (GLdouble)width/(GLdouble)height;

	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45, aspect, 0.1, 500);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glDrawBuffer(GL_BACK);

	glEnable(GL_DEPTH_TEST);

	glEnable(GL_LIGHTING);

	GLfloat lightZeroPosition[] = {2.0f, -2.0f, -8.0f, 1.0f};
	GLfloat lightZeroColor[] = {0.8f, 1.0f, 0.8f, 1.0f}; 

	glLightfv(GL_LIGHT0, GL_POSITION, lightZeroPosition);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightZeroColor);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightZeroColor);
	glEnable(GL_LIGHT0);

	GLfloat white[] = {1.0f, 1.0f, 1.0f, 1.0f};

	glMaterialfv(GL_FRONT, GL_DIFFUSE, white);
	glMaterialfv(GL_FRONT, GL_SPECULAR, white);
	glMaterialf(GL_FRONT, GL_SHININESS, 20.0f);

	m_opengl_init = true;
}

void CTutorialView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	// TODO: Add your message handler code here
	static DWORD last_t = GetTickCount();
	DWORD t = GetTickCount() - last_t;
	if (t == 0)
		return;
	last_t = GetTickCount();

	static double tt = 0;

	tt += t;

	glMatrixMode(GL_MODELVIEW);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
		gluLookAt(0, 0, 0,
				  0, 0, -1,
				  0, 1, 0);

/*		gluLookAt(CUBE_R*NB_CUBE_X/2, 30, 0,
				  CUBE_R*NB_CUBE_X/2, 29.4, -1,
				  0, 1, 0);
		float zz=float(2*CUBE_R * (3 + 2*sin(tt/3000)));
		for (int x = 0 ; x < NB_CUBE_X ; x++)
			for (int y = 0 ; y < NB_CUBE_Y ; y++)
				for (int z = 0 ; z < NB_CUBE_Z ; z++)
				{
					Cube[x][y][z]->Update(t);
					Cube[x][y][z]->Draw(0,0,zz,true);
				}*/
/*		static GLfloat ctrlpoints[4][3] = {
				{-4.0, -4.0, -15.0}, {-2.0, 4.0, -15.0},
				{ 2.0, -4.0, -15.0}, { 4.0, 4.0, -15.0}};
		static GLfloat ctrlpoints2[4][4] = {
				{ 1.0, 0.0, 0.0, 1.0}, { 0.0, 1.0, 0.0,1.0},
				{ 0.0, 0.0, 1.0, 1.0}, { 1.0, 1.0, 1.0,1.0}};
		ctrlpoints2[0][0] = float(0.5 + sin(tt/500)/2);
		glMap1f(GL_MAP1_VERTEX_3, 0.0, 1.0, 3, 4, &ctrlpoints[0][0]);
		glMap1f(GL_MAP1_COLOR_4, 0.0, 1.0, 4, 4, &ctrlpoints2[0][0]);
		glEnable(GL_MAP1_VERTEX_3);
		glEnable(GL_MAP1_COLOR_4);
		glBegin(GL_LINE_STRIP);
		for (GLfloat i = 0.0; i <= 1; i+=0.01f) 
			glEvalCoord1f(i);
		glEnd();
		glDisable(GL_MAP1_VERTEX_3);
		glDisable(GL_MAP1_COLOR_4);

		glPointSize(5.0);
		glColor3f(1.0, 1.0, 0.0);
		glBegin(GL_POINTS);
		for (int j = 0; j < 4; j++) 
			glVertex3fv(&ctrlpoints[j][0]);
		glEnd();*/
//		glEnable(GL_LINE_SMOOTH);
		static GLfloat ctrlpoints[4][4][3] = {
			{{-1.5, -1.5,  4.0}, {-0.5, -1.5,  2.0},
			{  0.5, -1.5, -1.0}, { 1.5, -1.5,  2.0}}, 
			{{-1.5, -0.5,  1.0}, {-0.5, -0.5,  3.0}, 
			{  0.5, -0.5,  0.0}, { 1.5, -0.5, -1.0}}, 
			{{-1.5,  0.5,  4.0}, {-0.5,  0.5,  0.0}, 
			{  0.5,  0.5,  3.0}, { 1.5,  0.5,  4.0}}, 
			{{-1.5,  1.5, -2.0}, {-0.5,  1.5, -2.0}, 
			{  0.5,  1.5,  0.0}, { 1.5,  1.5, -1.0}}
		};
		static GLfloat texpts[2][2][2] = {{{0.0, 0.0}, {0.0, 1.0}}, {{1.0, 0.0}, {1.0, 1.0}}};

		static CTexture texture;
		static GLuint texture_num;

		if (!texture.GetData())
		{
			texture.ReadFile("Marbre.bmp");
			texture.BGRtoRGB();
			glGenTextures(1, &texture_num);
			glBindTexture(GL_TEXTURE_2D, texture_num);
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, texture.GetWidth(), texture.GetHeight(), GL_RGB, GL_UNSIGNED_BYTE, texture.GetData());
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
			glEnable(GL_TEXTURE_2D);
		}
		glMap2f(GL_MAP2_VERTEX_3, 0, 1, 3, 4, 0, 1, 12, 4, &ctrlpoints[0][0][0]);
		glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, &texpts[0][0][0]);

		glEnable(GL_MAP2_VERTEX_3);
		glEnable(GL_MAP2_TEXTURE_COORD_2);
		glEnable(GL_AUTO_NORMAL);

		glTranslatef(0,0,-12.0f);
		glRotatef(45, 1, 0, 0);
		glRotatef(float(180.0*sin(tt/1000)), 0.0, 0.0, 1.0);
		glColor3f(1.0, 1.0, 1.0);
/*		for (int j = 0; j <= 8; j++) 
		{
			glBegin(GL_LINE_STRIP);
			for (int i = 0; i <= 30; i++)
				glEvalCoord2f((GLfloat)i/30.0f, (GLfloat)j/8.0f);
			glEnd();
			glBegin(GL_LINE_STRIP);
			for (i = 0; i <= 30; i++)
				glEvalCoord2f((GLfloat)j/8.0f, (GLfloat)i/30.0f);
			glEnd();
		}*/
		glMapGrid2f(50, 0.0, 1.0, 50, 0.0, 1.0);
		glEvalMesh2(GL_FILL, 0, 50, 0, 50);
		glDisable(GL_MAP2_VERTEX_3);
		glDisable(GL_MAP2_TEXTURE_COORD_2);

	glPopMatrix();

	glFlush();

	SwapBuffers(dc.m_ps.hdc);
	// Do not call CView::OnPaint() for painting messages
}
